@@ -1,7 +1,7 @@
 /*
     datagencli.c
     compressible data command line generator
-    Copyright (C) Yann Collet 2012-2020
+    Copyright (C) Yann Collet 2012-2015
 
     GPL v2 License
 
@@ -20,47 +20,56 @@
     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
     You can contact the author at :
-   - LZ4 source repository : https://github.com/lz4/lz4
-   - Public forum : https://groups.google.com/forum/#!forum/lz4c
+   - LZ5 source repository : https://github.com/inikep/lz5
 */
 
-
 /**************************************
 *  Includes
 **************************************/
-#include "util.h"      /* U32 */
 #include <stdio.h>     /* fprintf, stderr */
 #include "datagen.h"   /* RDG_generate */
-#include "loremOut.h"  /* LOREM_genOut */
-#include "lz4.h"       /* LZ4_VERSION_STRING */
 
 
 /**************************************
-*  Compiler specific
+*  Basic Types
 **************************************/
-#ifdef _MSC_VER    /* Visual Studio */
-# pragma warning(disable : 4127)    /* disable: C4127: conditional expression is constant */
-# define strtoull  _strtoui64  /* https://docs.microsoft.com/en-us/cpp/c-runtime-library/reference/strtoui64-wcstoui64-strtoui64-l-wcstoui64-l */
+#if defined (__STDC_VERSION__) && (__STDC_VERSION__ >= 199901L)   /* C99 */
+# include <stdint.h>
+  typedef  uint8_t BYTE;
+  typedef uint16_t U16;
+  typedef uint32_t U32;
+  typedef  int32_t S32;
+  typedef uint64_t U64;
+#else
+  typedef unsigned char       BYTE;
+  typedef unsigned short      U16;
+  typedef unsigned int        U32;
+  typedef   signed int        S32;
+  typedef unsigned long long  U64;
 #endif
 
 
 /**************************************
 *  Constants
 **************************************/
+#ifndef ZSTD_VERSION
+#  define ZSTD_VERSION "r1"
+#endif
+
 #define KB *(1 <<10)
 #define MB *(1 <<20)
 #define GB *(1U<<30)
 
 #define SIZE_DEFAULT (64 KB)
 #define SEED_DEFAULT 0
-#define COMPRESSIBILITY_NOT_SET 9999
+#define COMPRESSIBILITY_DEFAULT 50
 
 
 /**************************************
 *  Macros
 **************************************/
 #define DISPLAY(...)         fprintf(stderr, __VA_ARGS__)
-#define DISPLAYLEVEL(l, ...) do { if (displayLevel>=(l)) DISPLAY(__VA_ARGS__); } while (0)
+#define DISPLAYLEVEL(l, ...) if (displayLevel>=l) { DISPLAY(__VA_ARGS__); }
 static unsigned displayLevel = 2;
 
 
@@ -76,7 +85,7 @@ static int usage(char* programName)
     DISPLAY( "Arguments :\n");
     DISPLAY( " -g#    : generate # data (default:%i)\n", SIZE_DEFAULT);
     DISPLAY( " -s#    : Select seed (default:%i)\n", SEED_DEFAULT);
-    DISPLAY( " -P#    : Select compressibility in %% (range [0-100])\n");
+    DISPLAY( " -P#    : Select compressibility in %% (default:%i%%)\n", COMPRESSIBILITY_DEFAULT);
     DISPLAY( " -h     : display help and exit\n");
     DISPLAY( "Special values :\n");
     DISPLAY( " -P0    : generate incompressible noise\n");
@@ -88,29 +97,39 @@ static int usage(char* programName)
 int main(int argc, char** argv)
 {
     int argNb;
-    unsigned long long proba = COMPRESSIBILITY_NOT_SET;
+    double proba = (double)COMPRESSIBILITY_DEFAULT / 100;
     double litProba = 0.0;
     U64 size = SIZE_DEFAULT;
-    unsigned seed = SEED_DEFAULT;
+    U32 seed = SEED_DEFAULT;
     char* programName;
 
     /* Check command line */
     programName = argv[0];
-    for(argNb=1; argNb<argc; argNb++) {
+    for(argNb=1; argNb<argc; argNb++)
+    {
         char* argument = argv[argNb];
 
         if(!argument) continue;   /* Protection if argument empty */
 
         /* Handle commands. Aggregated commands are allowed */
-        if (*argument=='-') {
+        if (*argument=='-')
+        {
             argument++;
-            while (*argument!=0) {
-                switch(*argument) {
+            while (*argument!=0)
+            {
+                switch(*argument)
+                {
                 case 'h':
                     return usage(programName);
                 case 'g':
                     argument++;
-                    size = strtoull(argument, &argument, 10);
+                    size=0;
+                    while ((*argument>='0') && (*argument<='9'))
+                    {
+                        size *= 10;
+                        size += *argument - '0';
+                        argument++;
+                    }
                     if (*argument=='K') { size <<= 10; argument++; }
                     if (*argument=='M') { size <<= 20; argument++; }
                     if (*argument=='G') { size <<= 30; argument++; }
@@ -118,15 +137,35 @@ int main(int argc, char** argv)
                     break;
                 case 's':
                     argument++;
-                    seed = (unsigned)strtoul(argument, &argument, 10);
+                    seed=0;
+                    while ((*argument>='0') && (*argument<='9'))
+                    {
+                        seed *= 10;
+                        seed += *argument - '0';
+                        argument++;
+                    }
                     break;
                 case 'P':
                     argument++;
-                    proba = strtoull(argument, &argument, 10);
+                    proba=0.0;
+                    while ((*argument>='0') && (*argument<='9'))
+                    {
+                        proba *= 10;
+                        proba += *argument - '0';
+                        argument++;
+                    }
+                    if (proba>100.) proba=100.;
+                    proba /= 100.;
                     break;
                 case 'L':   /* hidden argument : Literal distribution probability */
                     argument++;
-                    litProba = (double) strtoull(argument, &argument, 10);
+                    litProba=0.;
+                    while ((*argument>='0') && (*argument<='9'))
+                    {
+                        litProba *= 10;
+                        litProba += *argument - '0';
+                        argument++;
+                    }
                     if (litProba>100.) litProba=100.;
                     litProba /= 100.;
                     break;
@@ -138,17 +177,15 @@ int main(int argc, char** argv)
                     return usage(programName);
                 }
             }
-        }  /* if (*argument=='-') */
-    }  /* for(argNb=1; argNb<argc; argNb++) */
 
-    DISPLAYLEVEL(4, "Data Generator %s \n", LZ4_VERSION_STRING);
-    DISPLAYLEVEL(3, "Seed = %u \n", seed);
-    if (proba != COMPRESSIBILITY_NOT_SET) {
-        DISPLAYLEVEL(3, "Compressibility : %i%%\n", (int)proba);
-        RDG_genOut(size, (double)proba / 100., litProba, seed);
-    } else {
-        LOREM_genOut(size, seed);
+        }
     }
+
+    DISPLAYLEVEL(4, "Data Generator %s \n", ZSTD_VERSION);
+    DISPLAYLEVEL(3, "Seed = %u \n", seed);
+    if (proba!=COMPRESSIBILITY_DEFAULT) DISPLAYLEVEL(3, "Compressibility : %i%%\n", (U32)(proba*100));
+
+    RDG_genOut(size, proba, litProba, seed);
     DISPLAYLEVEL(1, "\n");
 
     return 0;
