@@ -1,6 +1,6 @@
 /*
     bench.c - Demo program to benchmark open-source compression algorithm
-    Copyright (C) Yann Collet 2012-2016
+    Copyright (C) Yann Collet 2012-2015
 
     GPL v2 License
 
@@ -19,10 +19,23 @@
     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
     You can contact the author at :
-    - LZ4 source repository : https://github.com/lz4/lz4
-    - LZ4 public forum : https://groups.google.com/forum/#!forum/lz4c
+    - LZ5 source repository : https://github.com/inikep/lz5
 */
 
+/**************************************
+*  Compiler Options
+**************************************/
+/* Disable some Visual warning messages */
+#define _CRT_SECURE_NO_WARNINGS
+#define _CRT_SECURE_NO_DEPRECATE     /* VS2005 */
+
+/* Unix Large Files support (>4GB) */
+#if (defined(__sun__) && (!defined(__LP64__)))   // Sun Solaris 32-bits requires specific definitions
+#  define _LARGEFILE_SOURCE
+#  define _FILE_OFFSET_BITS 64
+#elif ! defined(__LP64__)                        // No point defining Large file for 64 bit
+#  define _LARGEFILE64_SOURCE
+#endif
 
 // S_ISREG & gettimeofday() are not supported by MSVC
 #if defined(_MSC_VER) || defined(_WIN32)
@@ -33,37 +46,41 @@
 /**************************************
 *  Includes
 **************************************/
-#include "platform.h"    /* _CRT_SECURE_NO_WARNINGS, Large Files support */
-#include "util.h"        /* U32, UTIL_getFileSize */
 #include <stdlib.h>      /* malloc, free */
-#include <stdio.h>       /* fprintf, fopen, ftello */
+#include <stdio.h>       /* fprintf, fopen, ftello64 */
 #include <sys/types.h>   /* stat64 */
 #include <sys/stat.h>    /* stat64 */
 #include <string.h>      /* strcmp */
 #include <time.h>        /* clock_t, clock(), CLOCKS_PER_SEC */
 
-#define LZ4_DISABLE_DEPRECATE_WARNINGS   /* LZ4_decompress_fast */
-#include "lz4.h"
-#include "lz4hc.h"
-#include "lz4frame.h"
+#include "lz5_compress.h"
+#include "lz5_decompress.h"
+#include "lz5_common.h"  /* LZ5_compress_MinLevel, LZ5_createStream_MinLevel */
+#include "lz5frame.h"
+
+#include "xxhash/xxhash.h"
+
+
+/**************************************
+*  Compiler Options
+**************************************/
+/* S_ISREG & gettimeofday() are not supported by MSVC */
+#if !defined(S_ISREG)
+#  define S_ISREG(x) (((x) & S_IFMT) == S_IFREG)
+#endif
 
-#include "xxhash.h"
 
 
 /**************************************
 *  Constants
 **************************************/
-#define PROGRAM_DESCRIPTION "LZ4 speed analyzer"
+#define PROGRAM_DESCRIPTION "LZ5 speed analyzer"
 #define AUTHOR "Yann Collet"
-#define WELCOME_MESSAGE "*** %s v%s %i-bits, by %s ***\n", PROGRAM_DESCRIPTION, LZ4_VERSION_STRING, (int)(sizeof(void*)*8), AUTHOR
+#define WELCOME_MESSAGE "*** %s v%s %i-bits, by %s ***\n", PROGRAM_DESCRIPTION, LZ5_VERSION_STRING, (int)(sizeof(void*)*8), AUTHOR
 
 #define NBLOOPS    6
 #define TIMELOOP   (CLOCKS_PER_SEC * 25 / 10)
 
-#define KB *(1 <<10)
-#define MB *(1 <<20)
-#define GB *(1U<<30)
-
 #define KNUTH      2654435761U
 #define MAX_MEM    (1920 MB)
 #define DEFAULT_CHUNKSIZE   (4 MB)
@@ -155,195 +172,149 @@ static size_t BMK_findMaxMem(U64 requiredMem)
 }
 
 
+static U64 BMK_GetFileSize(const char* infilename)
+{
+    int r;
+#if defined(_MSC_VER)
+    struct _stat64 statbuf;
+    r = _stat64(infilename, &statbuf);
+#else
+    struct stat statbuf;
+    r = stat(infilename, &statbuf);
+#endif
+    if (r || !S_ISREG(statbuf.st_mode)) return 0;   /* No good... */
+    return (U64)statbuf.st_size;
+}
+
+
 /*********************************************************
 *  Benchmark function
 *********************************************************/
-static LZ4_stream_t LZ4_stream;
-static void local_LZ4_resetDictT(void)
+LZ5_stream_t *LZ5_stream;
+static void local_LZ5_createStream(void)
 {
-    void* const r = LZ4_initStream(&LZ4_stream, sizeof(LZ4_stream));
-    assert(r != NULL); (void)r;
+    LZ5_stream = LZ5_resetStream_MinLevel(LZ5_stream);
 }
 
-static void local_LZ4_createStream(void)
-{
-    void* const r = LZ4_initStream(&LZ4_stream, sizeof(LZ4_stream));
-    assert(r != NULL); (void)r;
-}
-
-static int local_LZ4_saveDict(const char* in, char* out, int inSize)
+static int local_LZ5_saveDict(const char* in, char* out, int inSize)
 {
     (void)in;
-    return LZ4_saveDict(&LZ4_stream, out, inSize);
+    return LZ5_saveDict(LZ5_stream, out, inSize);
 }
 
-static int local_LZ4_compress_default_large(const char* in, char* out, int inSize)
+static int local_LZ5_compress_default_large(const char* in, char* out, int inSize)
 {
-    return LZ4_compress_default(in, out, inSize, LZ4_compressBound(inSize));
+    return LZ5_compress_MinLevel(in, out, inSize, LZ5_compressBound(inSize));
 }
 
-static int local_LZ4_compress_default_small(const char* in, char* out, int inSize)
+static int local_LZ5_compress_default_small(const char* in, char* out, int inSize)
 {
-    return LZ4_compress_default(in, out, inSize, LZ4_compressBound(inSize)-1);
+    return LZ5_compress_MinLevel(in, out, inSize, LZ5_compressBound(inSize)-1);
 }
 
-static int local_LZ4_compress_destSize(const char* in, char* out, int inSize)
+static int local_LZ5_compress_withState(const char* in, char* out, int inSize)
 {
-    return LZ4_compress_destSize(in, out, &inSize, LZ4_compressBound(inSize)-1);
+    return LZ5_compress_extState_MinLevel(LZ5_stream, in, out, inSize, LZ5_compressBound(inSize));
 }
 
-static int local_LZ4_compress_fast0(const char* in, char* out, int inSize)
+static int local_LZ5_compress_limitedOutput_withState(const char* in, char* out, int inSize)
 {
-    return LZ4_compress_fast(in, out, inSize, LZ4_compressBound(inSize), 0);
+    return LZ5_compress_extState_MinLevel(LZ5_stream, in, out, inSize, LZ5_compressBound(inSize)-1);
 }
 
-static int local_LZ4_compress_fast1(const char* in, char* out, int inSize)
+static int local_LZ5_compress_continue(const char* in, char* out, int inSize)
 {
-    return LZ4_compress_fast(in, out, inSize, LZ4_compressBound(inSize), 1);
+    return LZ5_compress_continue(LZ5_stream, in, out, inSize, LZ5_compressBound(inSize));
 }
 
-static int local_LZ4_compress_fast2(const char* in, char* out, int inSize)
+static int local_LZ5_compress_limitedOutput_continue(const char* in, char* out, int inSize)
 {
-    return LZ4_compress_fast(in, out, inSize, LZ4_compressBound(inSize), 2);
+    return LZ5_compress_continue(LZ5_stream, in, out, inSize, LZ5_compressBound(inSize)-1);
 }
 
-static int local_LZ4_compress_fast17(const char* in, char* out, int inSize)
-{
-    return LZ4_compress_fast(in, out, inSize, LZ4_compressBound(inSize), 17);
-}
-
-static int local_LZ4_compress_fast_extState0(const char* in, char* out, int inSize)
-{
-    return LZ4_compress_fast_extState(&LZ4_stream, in, out, inSize, LZ4_compressBound(inSize), 0);
-}
-
-static int local_LZ4_compress_fast_continue0(const char* in, char* out, int inSize)
-{
-    return LZ4_compress_fast_continue(&LZ4_stream, in, out, inSize, LZ4_compressBound(inSize), 0);
-}
-
-#ifndef LZ4_DLL_IMPORT
-#if defined (__cplusplus)
-extern "C" {
-#endif
-
-/* declare hidden function */
-extern int LZ4_compress_forceExtDict (LZ4_stream_t* LZ4_stream, const char* source, char* dest, int inputSize);
-
-#if defined (__cplusplus)
-}
-#endif
-
-static int local_LZ4_compress_forceDict(const char* in, char* out, int inSize)
-{
-    return LZ4_compress_forceExtDict(&LZ4_stream, in, out, inSize);
-}
-#endif
-
 
 /* HC compression functions */
-LZ4_streamHC_t LZ4_streamHC;
-static void local_LZ4_resetStreamHC(void)
+LZ5_stream_t* LZ5_streamPtr;
+static void local_LZ5_resetStream(void)
 {
-    LZ4_initStreamHC(&LZ4_streamHC, sizeof(LZ4_streamHC));
+    LZ5_streamPtr = LZ5_resetStream(LZ5_streamPtr, 0);
 }
 
-static int local_LZ4_saveDictHC(const char* in, char* out, int inSize)
+static int local_LZ5_saveDictHC(const char* in, char* out, int inSize)
 {
     (void)in;
-    return LZ4_saveDictHC(&LZ4_streamHC, out, inSize);
+    return LZ5_saveDict(LZ5_streamPtr, out, inSize);
 }
 
-static int local_LZ4_compress_HC(const char* in, char* out, int inSize)
+static int local_LZ5_compress_extState(const char* in, char* out, int inSize)
 {
-    return LZ4_compress_HC(in, out, inSize, LZ4_compressBound(inSize), 9);
+    return LZ5_compress_extState(LZ5_streamPtr, in, out, inSize, LZ5_compressBound(inSize), 0);
 }
 
-static int local_LZ4_compress_HC_extStateHC(const char* in, char* out, int inSize)
+static int local_LZ5_compress_extState_limitedOutput(const char* in, char* out, int inSize)
 {
-    return LZ4_compress_HC_extStateHC(&LZ4_streamHC, in, out, inSize, LZ4_compressBound(inSize), 9);
+    return LZ5_compress_extState(LZ5_streamPtr, in, out, inSize, LZ5_compressBound(inSize)-1, 0);
 }
 
-static int local_LZ4_compress_HC_continue(const char* in, char* out, int inSize)
+static int local_LZ5_compress(const char* in, char* out, int inSize)
 {
-    return LZ4_compress_HC_continue(&LZ4_streamHC, in, out, inSize, LZ4_compressBound(inSize));
+    return LZ5_compress(in, out, inSize, LZ5_compressBound(inSize), 0);
+}
+
+static int local_LZ5_compress_limitedOutput(const char* in, char* out, int inSize)
+{
+    return LZ5_compress(in, out, inSize, LZ5_compressBound(inSize)-1, 0);
+}
+
+static int local_LZ5_compressHC_continue(const char* in, char* out, int inSize)
+{
+    return LZ5_compress_continue(LZ5_streamPtr, in, out, inSize, LZ5_compressBound(inSize));
+}
+
+static int local_LZ5_compress_continue_limitedOutput(const char* in, char* out, int inSize)
+{
+    return LZ5_compress_continue(LZ5_streamPtr, in, out, inSize, LZ5_compressBound(inSize)-1);
 }
 
 
 /* decompression functions */
-static int local_LZ4_decompress_fast(const char* in, char* out, int inSize, int outSize)
+static int local_LZ5_decompress_safe_usingDict(const char* in, char* out, int inSize, int outSize)
 {
     (void)inSize;
-    LZ4_decompress_fast(in, out, outSize);
+    LZ5_decompress_safe_usingDict(in, out, inSize, outSize, out - 65536, 65536);
     return outSize;
 }
 
-static int local_LZ4_decompress_fast_usingDict_prefix(const char* in, char* out, int inSize, int outSize)
+extern int LZ5_decompress_safe_forceExtDict(const char* in, char* out, int inSize, int outSize, const char* dict, int dictSize);
+
+static int local_LZ5_decompress_safe_forceExtDict(const char* in, char* out, int inSize, int outSize)
 {
     (void)inSize;
-    LZ4_decompress_fast_usingDict(in, out, outSize, out - 65536, 65536);
+    LZ5_decompress_safe_forceExtDict(in, out, inSize, outSize, out - 65536, 65536);
     return outSize;
 }
 
-static int local_LZ4_decompress_fast_usingExtDict(const char* in, char* out, int inSize, int outSize)
+static int local_LZ5_decompress_safe_partial(const char* in, char* out, int inSize, int outSize)
 {
-    (void)inSize;
-    LZ4_decompress_fast_usingDict(in, out, outSize, out - 65536, 65535);
-    return outSize;
-}
-
-static int local_LZ4_decompress_safe_usingDict(const char* in, char* out, int inSize, int outSize)
-{
-    (void)inSize;
-    LZ4_decompress_safe_usingDict(in, out, inSize, outSize, out - 65536, 65536);
-    return outSize;
-}
-
-#ifndef LZ4_DLL_IMPORT
-#if defined (__cplusplus)
-extern "C" {
-#endif
-
-extern int LZ4_decompress_safe_forceExtDict(const char* in, char* out, int inSize, int outSize, const void* dict, size_t dictSize);
-
-#if defined (__cplusplus)
-}
-#endif
-
-static int local_LZ4_decompress_safe_forceExtDict(const char* in, char* out, int inSize, int outSize)
-{
-    (void)inSize;
-    LZ4_decompress_safe_forceExtDict(in, out, inSize, outSize, out - 65536, 65536);
-    return outSize;
-}
-#endif
-
-static int local_LZ4_decompress_safe_partial(const char* in, char* out, int inSize, int outSize)
-{
-    int result = LZ4_decompress_safe_partial(in, out, inSize, outSize - 5, outSize);
-    if (result < 0) return result;
-    return outSize;
+    return LZ5_decompress_safe_partial(in, out, inSize, outSize - 5, outSize);
 }
 
 
 /* frame functions */
-static int local_LZ4F_compressFrame(const char* in, char* out, int inSize)
+static int local_LZ5F_compressFrame(const char* in, char* out, int inSize)
 {
-    assert(inSize >= 0);
-    return (int)LZ4F_compressFrame(out, LZ4F_compressFrameBound((size_t)inSize, NULL), in, (size_t)inSize, NULL);
+    return (int)LZ5F_compressFrame(out, 2*inSize + 16, in, inSize, NULL);
 }
 
-static LZ4F_decompressionContext_t g_dCtx;
+static LZ5F_decompressionContext_t g_dCtx;
 
-static int local_LZ4F_decompress(const char* in, char* out, int inSize, int outSize)
+static int local_LZ5F_decompress(const char* in, char* out, int inSize, int outSize)
 {
-    size_t srcSize = (size_t)inSize;
-    size_t dstSize = (size_t)outSize;
+    size_t srcSize = inSize;
+    size_t dstSize = outSize;
     size_t result;
-    assert(inSize >= 0);
-    assert(outSize >= 0);
-    result = LZ4F_decompress(g_dCtx, out, &dstSize, in, &srcSize, NULL);
-    if (result!=0) { DISPLAY("Error decompressing frame : unfinished frame\n"); exit(8); }
+    result = LZ5F_decompress(g_dCtx, out, &dstSize, in, &srcSize, NULL);
+    if (result!=0) { DISPLAY("Error decompressing frame : unfinished frame (%d)\n", (int)result); exit(8); }
     if (srcSize != (size_t)inSize) { DISPLAY("Error decompressing frame : read size incorrect\n"); exit(9); }
     return (int)dstSize;
 }
@@ -356,8 +327,14 @@ int fullSpeedBench(const char** fileNamesTable, int nbFiles)
     int fileIdx=0;
 
     /* Init */
-    { size_t const errorCode = LZ4F_createDecompressionContext(&g_dCtx, LZ4F_VERSION);
-      if (LZ4F_isError(errorCode)) { DISPLAY("dctx allocation issue \n"); return 10; } }
+    { size_t const errorCode = LZ5F_createDecompressionContext(&g_dCtx, LZ5F_VERSION);
+      if (LZ5F_isError(errorCode)) { DISPLAY("dctx allocation issue \n"); return 10; } }
+
+  LZ5_streamPtr = LZ5_createStream(0);
+  if (!LZ5_streamPtr) { DISPLAY("LZ5_streamPtr allocation issue \n"); return 10; }
+
+  LZ5_stream = LZ5_createStream_MinLevel();
+  if (!LZ5_stream) { DISPLAY("LZ5_stream allocation issue \n"); return 10; }
 
     /* Loop for each fileName */
     while (fileIdx<nbFiles) {
@@ -373,12 +350,13 @@ int fullSpeedBench(const char** fileNamesTable, int nbFiles)
       size_t readSize;
       int compressedBuffSize;
       U32 crcOriginal;
+      size_t errorCode;
 
       /* Check file existence */
       if (inFile==NULL) { DISPLAY( "Pb opening %s\n", inFileName); return 11; }
 
       /* Memory size adjustments */
-      inFileSize = UTIL_getFileSize(inFileName);
+      inFileSize = BMK_GetFileSize(inFileName);
       if (inFileSize==0) { DISPLAY( "file is empty\n"); fclose(inFile); return 11; }
       benchedSize = BMK_findMaxMem(inFileSize*2) / 2;   /* because 2 buffers */
       if (benchedSize==0) { DISPLAY( "not enough memory\n"); fclose(inFile); return 11; }
@@ -389,8 +367,8 @@ int fullSpeedBench(const char** fileNamesTable, int nbFiles)
       /* Allocation */
       chunkP = (struct chunkParameters*) malloc(((benchedSize / (size_t)g_chunkSize)+1) * sizeof(struct chunkParameters));
       orig_buff = (char*) malloc(benchedSize);
-      nbChunks = (int) ((benchedSize + (size_t)g_chunkSize - 1) / (size_t)g_chunkSize);
-      maxCompressedChunkSize = LZ4_compressBound(g_chunkSize);
+      nbChunks = (int) ((benchedSize + (g_chunkSize-1)) / g_chunkSize);
+      maxCompressedChunkSize = LZ5_compressBound(g_chunkSize);
       compressedBuffSize = nbChunks * maxCompressedChunkSize;
       compressed_buff = (char*)malloc((size_t)compressedBuffSize);
       if(!chunkP || !orig_buff || !compressed_buff) {
@@ -444,7 +422,7 @@ int fullSpeedBench(const char** fileNamesTable, int nbFiles)
                 char* out = compressed_buff;
                 nbChunks = (int) (((int)benchedSize + (g_chunkSize-1))/ g_chunkSize);
                 for (i=0; i<nbChunks; i++) {
-                    chunkP[i].id = (U32)i;
+                    chunkP[i].id = i;
                     chunkP[i].origBuffer = in; in += g_chunkSize;
                     if ((int)remaining > g_chunkSize) { chunkP[i].origSize = g_chunkSize; remaining -= g_chunkSize; } else { chunkP[i].origSize = (int)remaining; remaining = 0; }
                     chunkP[i].compressedBuffer = out; out += maxCompressedChunkSize;
@@ -455,33 +433,29 @@ int fullSpeedBench(const char** fileNamesTable, int nbFiles)
             switch(cAlgNb)
             {
             case 0 : DISPLAY("Compression functions : \n"); continue;
-            case 1 : compressionFunction = local_LZ4_compress_default_large; compressorName = "LZ4_compress_default"; break;
-            case 2 : compressionFunction = local_LZ4_compress_default_small; compressorName = "LZ4_compress_default(small dst)"; break;
-            case 3 : compressionFunction = local_LZ4_compress_destSize; compressorName = "LZ4_compress_destSize"; break;
-            case 4 : compressionFunction = local_LZ4_compress_fast0; compressorName = "LZ4_compress_fast(0)"; break;
-            case 5 : compressionFunction = local_LZ4_compress_fast1; compressorName = "LZ4_compress_fast(1)"; break;
-            case 6 : compressionFunction = local_LZ4_compress_fast2; compressorName = "LZ4_compress_fast(2)"; break;
-            case 7 : compressionFunction = local_LZ4_compress_fast17; compressorName = "LZ4_compress_fast(17)"; break;
-            case 8 : compressionFunction = local_LZ4_compress_fast_extState0; compressorName = "LZ4_compress_fast_extState(0)"; break;
-            case 9 : compressionFunction = local_LZ4_compress_fast_continue0; initFunction = local_LZ4_createStream; compressorName = "LZ4_compress_fast_continue(0)"; break;
+            case 1 : compressionFunction = local_LZ5_compress_default_large; compressorName = "LZ5_compress_MinLevel"; break;
+            case 2 : compressionFunction = local_LZ5_compress_default_small; compressorName = "LZ5_compress_MinLevel(small dst)"; break;
 
-            case 10: compressionFunction = local_LZ4_compress_HC; compressorName = "LZ4_compress_HC"; break;
-            case 12: compressionFunction = local_LZ4_compress_HC_extStateHC; compressorName = "LZ4_compress_HC_extStateHC"; break;
-            case 14: compressionFunction = local_LZ4_compress_HC_continue; initFunction = local_LZ4_resetStreamHC; compressorName = "LZ4_compress_HC_continue"; break;
-#ifndef LZ4_DLL_IMPORT
-            case 20: compressionFunction = local_LZ4_compress_forceDict; initFunction = local_LZ4_resetDictT; compressorName = "LZ4_compress_forceDict"; break;
-#endif
-            case 30: compressionFunction = local_LZ4F_compressFrame; compressorName = "LZ4F_compressFrame";
+            case 10: compressionFunction = local_LZ5_compress; compressorName = "LZ5_compress"; break;
+            case 11: compressionFunction = local_LZ5_compress_limitedOutput; compressorName = "LZ5_compress limitedOutput"; break;
+            case 12 : compressionFunction = local_LZ5_compress_extState; compressorName = "LZ5_compress_extState"; break;
+            case 13: compressionFunction = local_LZ5_compress_extState_limitedOutput; compressorName = "LZ5_compress_extState limitedOutput"; break;
+            case 14: compressionFunction = local_LZ5_compressHC_continue; initFunction = local_LZ5_resetStream; compressorName = "LZ5_compress_continue"; break;
+            case 15: compressionFunction = local_LZ5_compress_continue_limitedOutput; initFunction = local_LZ5_resetStream; compressorName = "LZ5_compress_continue limitedOutput"; break;
+            case 30: compressionFunction = local_LZ5F_compressFrame; compressorName = "LZ5F_compressFrame";
                         chunkP[0].origSize = (int)benchedSize; nbChunks=1;
                         break;
-            case 40: compressionFunction = local_LZ4_saveDict; compressorName = "LZ4_saveDict";
-                        if (chunkP[0].origSize < 8) { DISPLAY(" cannot bench %s with less then 8 bytes \n", compressorName); continue; }
-                        LZ4_loadDict(&LZ4_stream, chunkP[0].origBuffer, chunkP[0].origSize);
+            case 40: compressionFunction = local_LZ5_saveDict; compressorName = "LZ5_saveDict";
+                        LZ5_loadDict(LZ5_stream, chunkP[0].origBuffer, chunkP[0].origSize);
                         break;
-            case 41: compressionFunction = local_LZ4_saveDictHC; compressorName = "LZ4_saveDictHC";
-                        if (chunkP[0].origSize < 8) { DISPLAY(" cannot bench %s with less then 8 bytes \n", compressorName); continue; }
-                        LZ4_loadDictHC(&LZ4_streamHC, chunkP[0].origBuffer, chunkP[0].origSize);
+            case 41: compressionFunction = local_LZ5_saveDictHC; compressorName = "LZ5_saveDict";
+                        LZ5_loadDict(LZ5_streamPtr, chunkP[0].origBuffer, chunkP[0].origSize);
                         break;
+            case 16: compressionFunction = local_LZ5_compress_withState; compressorName = "LZ5_compress_extState_MinLevel(1)"; break;
+            case 17: compressionFunction = local_LZ5_compress_limitedOutput_withState; compressorName = "LZ5_compress_extState_MinLevel(1) limitedOutput"; break;
+            case 18: compressionFunction = local_LZ5_compress_continue; initFunction = local_LZ5_createStream; compressorName = "LZ5_compress_continue(1)"; break;
+            case 19: compressionFunction = local_LZ5_compress_limitedOutput_continue; initFunction = local_LZ5_createStream; compressorName = "LZ5_compress_continue(1) limitedOutput"; break;
+            case 60: DISPLAY("Obsolete compression functions : \n"); continue;
             default :
                 continue;   /* unknown ID : just skip */
             }
@@ -490,7 +464,7 @@ int fullSpeedBench(const char** fileNamesTable, int nbFiles)
                 double averageTime;
                 clock_t clockTime;
 
-                PROGRESS("%2i-%-34.34s :%10i ->\r", loopNb, compressorName, (int)benchedSize);
+                PROGRESS("%1i- %-28.28s :%9i ->\r", loopNb, compressorName, (int)benchedSize);
                 { size_t i; for (i=0; i<benchedSize; i++) compressed_buff[i]=(char)i; }     /* warming up memory */
 
                 nb_loops = 0;
@@ -501,8 +475,7 @@ int fullSpeedBench(const char** fileNamesTable, int nbFiles)
                     if (initFunction!=NULL) initFunction();
                     for (chunkNb=0; chunkNb<nbChunks; chunkNb++) {
                         chunkP[chunkNb].compressedSize = compressionFunction(chunkP[chunkNb].origBuffer, chunkP[chunkNb].compressedBuffer, chunkP[chunkNb].origSize);
-                        if (chunkP[chunkNb].compressedSize==0)
-                            DISPLAY("ERROR ! %s() = 0 !! \n", compressorName), exit(1);
+                        if (chunkP[chunkNb].compressedSize==0) DISPLAY("ERROR ! %s() = 0 !! \n", compressorName), exit(1);
                     }
                     nb_loops++;
                 }
@@ -513,13 +486,13 @@ int fullSpeedBench(const char** fileNamesTable, int nbFiles)
                 if (averageTime < bestTime) bestTime = averageTime;
                 cSize=0; for (chunkNb=0; chunkNb<nbChunks; chunkNb++) cSize += chunkP[chunkNb].compressedSize;
                 ratio = (double)cSize/(double)benchedSize*100.;
-                PROGRESS("%2i-%-34.34s :%10i ->%9i (%5.2f%%),%7.1f MB/s\r", loopNb, compressorName, (int)benchedSize, (int)cSize, ratio, (double)benchedSize / bestTime / 1000000);
+                PROGRESS("%1i- %-28.28s :%9i ->%9i (%5.2f%%),%7.1f MB/s\r", loopNb, compressorName, (int)benchedSize, (int)cSize, ratio, (double)benchedSize / bestTime / 1000000);
             }
 
             if (ratio<100.)
-                DISPLAY("%2i-%-34.34s :%10i ->%9i (%5.2f%%),%7.1f MB/s\n", cAlgNb, compressorName, (int)benchedSize, (int)cSize, ratio, (double)benchedSize / bestTime / 1000000);
+                DISPLAY("%2i-%-28.28s :%9i ->%9i (%5.2f%%),%7.1f MB/s\n", cAlgNb, compressorName, (int)benchedSize, (int)cSize, ratio, (double)benchedSize / bestTime / 1000000);
             else
-                DISPLAY("%2i-%-34.34s :%10i ->%9i (%5.1f%%),%7.1f MB/s\n", cAlgNb, compressorName, (int)benchedSize, (int)cSize, ratio, (double)benchedSize / bestTime / 100000);
+                DISPLAY("%2i-%-28.28s :%9i ->%9i (%5.1f%%),%7.1f MB/s\n", cAlgNb, compressorName, (int)benchedSize, (int)cSize, ratio, (double)benchedSize / bestTime / 100000);
         }
 
         /* Prepare layout for decompression */
@@ -539,35 +512,28 @@ int fullSpeedBench(const char** fileNamesTable, int nbFiles)
           }
         }
         for (chunkNb=0; chunkNb<nbChunks; chunkNb++) {
-            chunkP[chunkNb].compressedSize = LZ4_compress_default(chunkP[chunkNb].origBuffer, chunkP[chunkNb].compressedBuffer, chunkP[chunkNb].origSize, maxCompressedChunkSize);
-            if (chunkP[chunkNb].compressedSize==0)
-                DISPLAY("ERROR ! %s() = 0 !! \n", "LZ4_compress"), exit(1);
+            chunkP[chunkNb].compressedSize = LZ5_compress_MinLevel(chunkP[chunkNb].origBuffer, chunkP[chunkNb].compressedBuffer, chunkP[chunkNb].origSize, LZ5_compressBound(chunkP[chunkNb].origSize));
+            if (chunkP[chunkNb].compressedSize==0) DISPLAY("ERROR ! %s() = 0 !! \n", "LZ5_compress_MinLevel"), exit(1);
         }
 
         /* Decompression Algorithms */
-        for (dAlgNb=0; (dAlgNb <= NB_DECOMPRESSION_ALGORITHMS) && g_decompressionTest; dAlgNb++) {
+        for (dAlgNb=0; (dAlgNb <= NB_DECOMPRESSION_ALGORITHMS) && (g_decompressionTest); dAlgNb++) {
             const char* dName;
             int (*decompressionFunction)(const char*, char*, int, int);
             double bestTime = 100000000.;
-            int checkResult = 1;
 
             if ((g_decompressionAlgo != ALL_DECOMPRESSORS) && (g_decompressionAlgo != dAlgNb)) continue;
 
             switch(dAlgNb)
             {
             case 0: DISPLAY("Decompression functions : \n"); continue;
-            case 1: decompressionFunction = local_LZ4_decompress_fast; dName = "LZ4_decompress_fast"; break;
-            case 2: decompressionFunction = local_LZ4_decompress_fast_usingDict_prefix; dName = "LZ4_decompress_fast_usingDict(prefix)"; break;
-            case 3: decompressionFunction = local_LZ4_decompress_fast_usingExtDict; dName = "LZ4_decompress_fast_using(Ext)Dict"; break;
-            case 4: decompressionFunction = LZ4_decompress_safe; dName = "LZ4_decompress_safe"; break;
-            case 6: decompressionFunction = local_LZ4_decompress_safe_usingDict; dName = "LZ4_decompress_safe_usingDict"; break;
-            case 7: decompressionFunction = local_LZ4_decompress_safe_partial; dName = "LZ4_decompress_safe_partial"; checkResult = 0; break;
-#ifndef LZ4_DLL_IMPORT
-            case 8: decompressionFunction = local_LZ4_decompress_safe_forceExtDict; dName = "LZ4_decompress_safe_forceExtDict"; break;
-#endif
-            case 9: decompressionFunction = local_LZ4F_decompress; dName = "LZ4F_decompress";
-                {   size_t const errorCode = LZ4F_compressFrame(compressed_buff, compressedBuffSize, orig_buff, benchedSize, NULL);
-                    if (LZ4F_isError(errorCode)) {
+            case 4: decompressionFunction = LZ5_decompress_safe; dName = "LZ5_decompress_safe"; break;
+            case 6: decompressionFunction = local_LZ5_decompress_safe_usingDict; dName = "LZ5_decompress_safe_usingDict"; break;
+            case 7: decompressionFunction = local_LZ5_decompress_safe_partial; dName = "LZ5_decompress_safe_partial"; break;
+            case 8: decompressionFunction = local_LZ5_decompress_safe_forceExtDict; dName = "LZ5_decompress_safe_forceExtDict"; break;
+            case 9: decompressionFunction = local_LZ5F_decompress; dName = "LZ5F_decompress";
+                    errorCode = LZ5F_compressFrame(compressed_buff, compressedBuffSize, orig_buff, benchedSize, NULL);
+                    if (LZ5F_isError(errorCode)) {
                         DISPLAY("Error while preparing compressed frame\n");
                         free(orig_buff);
                         free(compressed_buff);
@@ -578,7 +544,6 @@ int fullSpeedBench(const char** fileNamesTable, int nbFiles)
                     chunkP[0].compressedSize = (int)errorCode;
                     nbChunks = 1;
                     break;
-                }
             default :
                 continue;   /* skip if unknown ID */
             }
@@ -590,7 +555,7 @@ int fullSpeedBench(const char** fileNamesTable, int nbFiles)
                 clock_t clockTime;
                 U32 crcDecoded;
 
-                PROGRESS("%2i-%-34.34s :%10i ->\r", loopNb, dName, (int)benchedSize);
+                PROGRESS("%1i- %-29.29s :%10i ->\r", loopNb, dName, (int)benchedSize);
 
                 nb_loops = 0;
                 clockTime = clock();
@@ -598,13 +563,9 @@ int fullSpeedBench(const char** fileNamesTable, int nbFiles)
                 clockTime = clock();
                 while(BMK_GetClockSpan(clockTime) < TIMELOOP) {
                     for (chunkNb=0; chunkNb<nbChunks; chunkNb++) {
-                        int const decodedSize = decompressionFunction(chunkP[chunkNb].compressedBuffer, chunkP[chunkNb].origBuffer,
-                                                                      chunkP[chunkNb].compressedSize, chunkP[chunkNb].origSize);
-                        if (chunkP[chunkNb].origSize != decodedSize) {
-                            DISPLAY("ERROR ! %s() == %i != %i !! \n",
-                                    dName, decodedSize, chunkP[chunkNb].origSize);
-                            exit(1);
-                    }   }
+                        int decodedSize = decompressionFunction(chunkP[chunkNb].compressedBuffer, chunkP[chunkNb].origBuffer, chunkP[chunkNb].compressedSize, chunkP[chunkNb].origSize);
+                        if (chunkP[chunkNb].origSize != decodedSize) DISPLAY("ERROR ! %s() == %i != %i !! \n", dName, decodedSize, chunkP[chunkNb].origSize), exit(1);
+                    }
                     nb_loops++;
                 }
                 clockTime = BMK_GetClockSpan(clockTime);
@@ -613,17 +574,14 @@ int fullSpeedBench(const char** fileNamesTable, int nbFiles)
                 averageTime = (double)clockTime / nb_loops / CLOCKS_PER_SEC;
                 if (averageTime < bestTime) bestTime = averageTime;
 
-                PROGRESS("%2i-%-34.34s :%10i -> %7.1f MB/s\r", loopNb, dName, (int)benchedSize, (double)benchedSize / bestTime / 1000000);
+                PROGRESS("%1i- %-29.29s :%10i -> %7.1f MB/s\r", loopNb, dName, (int)benchedSize, (double)benchedSize / bestTime / 1000000);
 
                 /* CRC Checking */
-                crcDecoded = XXH32(orig_buff, benchedSize, 0);
-                if (checkResult && (crcOriginal!=crcDecoded)) {
-                    DISPLAY("\n!!! WARNING !!! %14s : Invalid Checksum : %x != %x\n",
-                            inFileName, (unsigned)crcOriginal, (unsigned)crcDecoded);
-                    exit(1);
-            }   }
+                crcDecoded = XXH32(orig_buff, (int)benchedSize, 0);
+                if (crcOriginal!=crcDecoded) { DISPLAY("\n!!! WARNING !!! %14s : Invalid Checksum : %x != %x\n", inFileName, (unsigned)crcOriginal, (unsigned)crcDecoded); exit(1); }
+            }
 
-            DISPLAY("%2i-%-34.34s :%10i -> %7.1f MB/s\n", dAlgNb, dName, (int)benchedSize, (double)benchedSize / bestTime / 1000000);
+            DISPLAY("%2i-%-29.29s :%10i -> %7.1f MB/s\n", dAlgNb, dName, (int)benchedSize, (double)benchedSize / bestTime / 1000000);
         }
       }
       free(orig_buff);
@@ -631,7 +589,9 @@ int fullSpeedBench(const char** fileNamesTable, int nbFiles)
       free(chunkP);
     }
 
-    LZ4F_freeDecompressionContext(g_dCtx);
+    LZ5_freeStream(LZ5_stream);
+    LZ5_freeStream(LZ5_streamPtr);
+    LZ5F_freeDecompressionContext(g_dCtx);
     if (g_pause) { printf("press enter...\n"); (void)getchar(); }
 
     return 0;
